%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Reading non-residuation analysis information

:- module(readAnalysis,
          [resInfo/2, retractResInfoFacts/0, readNonResInfoForModule/2]).

:- use_module(prologbasics).
:- use_module(basics).
:- ensure_lib_loaded(prim_readshowterm).  % for term en/decoding

:- (swi7orHigher -> set_prolog_flag(double_quotes, codes) ; true).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Storage for non-residuation information
:- dynamic resInfo/2.

storeNonResInfo([]).
storeNonResInfo([(Func,NRArgs)|NRI]) :-
        assert(resInfo(Func,NRArgs)),
        storeNonResInfo(NRI).

retractResInfoFacts :- retract(resInfo(_,_)), fail.
retractResInfoFacts.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% read/generate the residuation info for module Mod w.r.t. given load path.
readNonResInfoForModule(LoadPath,Mod) :-
        readNonResInfoForModule(LoadPath,LoadPath,Mod).

readNonResInfoForModule(LoadPath,[Dir|Dirs],Mod) :-
	appendAtoms([Dir,'/',Mod],DirMod),
	prog2FlatCurryFile(DirMod,DirFcyFile),
	(existsFile(DirFcyFile)
         -> hierarchical2dirs(Mod,ModD),
            appendAtoms([Dir,'/.curry/pakcs/',ModD,'.resinfo'],ResInfoFile),
            tryReadNonResInfo(LoadPath,Mod,DirFcyFile,ResInfoFile)
	  ; readNonResInfoForModule(LoadPath,Dirs,Mod)).
readNonResInfoForModule(_,[],Mod) :-
        writeErr('Internal error: residuation info for module "'),
        writeErr(Mod), writeLnErr('" not found!'), fail.
        
tryReadNonResInfo(_,_,FcyFile,ResInfoFile) :-
        fileExistsAndNewer(ResInfoFile,FcyFile), !,
        readNonResInfo(ResInfoFile).
tryReadNonResInfo(LoadPath,Mod,FcyFile,ResInfoFile) :-
        invokeResiduationAnalyzer(LoadPath,Mod,ResInfoFile),
        fileExistsAndNewer(ResInfoFile,FcyFile), !,
        readNonResInfo(ResInfoFile).
tryReadNonResInfo(_,Mod,_,_) :-
        writeErr('WARNING: could not generate non-residuation info for "'),
        writeErr(Mod), writeLnErr('"').

readNonResInfo(FileName) :-
	(verbosityIntermediate
	 -> writeErr('>>> Reading '),
	    writeErr(FileName), writeErr(' ... '),
	    getRunTime(RT1) ; true),
	open(FileName,read,Stream),
	readStreamContents(Stream,TermString),
	user:readTerm(TermString,unchecked,Tail,Term),
	user:skipWhiteSpace(Tail,[]),
	(verbosityIntermediate
	 -> getRunTime(RT2),
	    RT is RT2-RT1,
	    writeErr(RT), writeLnErr(' ms.')
	  ; true),
        transNonResTerm(Term,NonResInfo), !,
	%writeLnErr(NonResInfo),
        storeNonResInfo(NonResInfo).
        %listing(resInfo/2).
readNonResInfo(FileName) :-
	writeErr('ERROR in readNonResInfo during reading of "'),
	writeErr(FileName), writeErr('"!'),
	nlErr, fail.

transNonResTerm(Xs,Ys) :- map2M(readAnalysis:transNRE,Xs,Ys).

transNRE('Prelude.(,)'(Name,Xs),(FName,Xs)) :-
        cp_string(Name,FNameS),
        atom_codes(FName,FNameS).


% Invoke the external residuation analysis tool to generate to
% the 'resinfo' file for the module.
invokeResiduationAnalyzer(LoadPath,Mod,ResInfoFile) :-
        Warn='Warning: tool "curry-anaresinfo" not found. Try: "cypm install residuation-analysis"',
        checkProgram('curry-anaresinfo',Warn,RIProg),
        appendAtoms(['"',RIProg,'" -o "',ResInfoFile,'" ',Mod],Cmd1),
        (verbosityIntermediate -> appendAtom(Cmd1,' -v2',Cmd2) ; Cmd2=Cmd1),
        getCmdWithLoadPath(LoadPath,Cmd2,Cmd),
        (verbosityIntermediate -> write('Executing: '), write(Cmd), nl ; true),
        shellCmdWithCurryPath(Cmd).
